/*******************************************************************************
   BH1900NUX.cpp

   Copyright (c) 2022 ROHM Co.,Ltd.

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*******************************************************************************/
#include <Arduino.h>
#include <Wire.h>
#include <BH1900NUX.h>

BH1900NUX::BH1900NUX(uint8_t addr_pin)
{
    if (addr_pin < BH1900NUX_SLAVE_ADDRESS_MAXNUM) {
        Address = BH1900NUX_SLAVE_ADDRESS_BASE + addr_pin;
    } else {
        Address = 0;
    }

    return;
}

BH1900NUX::~BH1900NUX()
{
    Address = 0;

    return;
}

int32_t BH1900NUX::init(void)
{
    int32_t result;

    result = init_check_val();
    if (result == BH1900NUX_OK) {
        (void)init_setting_ic();
        result = setting_wait(BH1900NUX_CONFIG_VAL);
    }

    return result;
}

int32_t BH1900NUX::start(void)
{
    int32_t  result;
    uint8_t  config[BH1900NUX_WRITE_DATA_SIZE];
    uint16_t temp;

    temp = BH1900NUX_CONFIG_VAL & BH1900NUX_CONFIG_MASK;
    config[0] = (uint8_t)((temp | BH1900NUX_CONFIG_SD_ACTIVE) >> 8);
    config[1] = (uint8_t)temp;

    result = write(BH1900NUX_CONFIG, config, sizeof(config));

    return result;
}

int32_t BH1900NUX::stop(void)
{
    int32_t  result;
    uint8_t  config[BH1900NUX_WRITE_DATA_SIZE];
    uint16_t temp;

    temp = BH1900NUX_CONFIG_VAL & BH1900NUX_CONFIG_MASK;
    config[0] = (uint8_t)((temp | BH1900NUX_CONFIG_SD_SLEEP) >> 8);
    config[1] = (uint8_t)temp;

    result = write(BH1900NUX_CONFIG, config, sizeof(config));

    return result;
}

int32_t BH1900NUX::get_val(uint8_t *raw)
{
    int32_t result;

    result = read(BH1900NUX_TEMPERATURE, raw, BH1900NUX_TEMPERATURE_DATA_SIZE);

    return result;
}

int32_t BH1900NUX::get_val(float32 *temp)
{
    int32_t result;
    uint8_t raw[BH1900NUX_TEMPERATURE_DATA_SIZE];
    int16_t data;

    result = get_val(raw);
    if (result == BH1900NUX_OK) {
        data  = ((int16_t)raw[0] << 8) | raw[1];
        *temp = (float32)data / BH1900NUX_CONVERT_TEMP;
        if ((*temp >= BH1900NUX_OPERATING_TEMP_MIN) && (*temp <= BH1900NUX_OPERATING_TEMP_MAX)) {
            result = BH1900NUX_OK;
        } else {
            result = BH1900NUX_TEMP_ERROR;
        }
    } else {
        result = BH1900NUX_COMM_ERROR;
    }

    return result;
}

float32 BH1900NUX::get_version(void)
{
    return BH1900NUX_DRIVER_VERSION;
}

int32_t BH1900NUX::init_check_val(void)
{
    int32_t result;
    uint8_t read_val = 0;

    result = read(BH1900NUX_SW_RESET, &read_val, sizeof(read_val));
    if (result == BH1900NUX_OK) {
        if (read_val == BH1900NUX_SW_RESET_DEFAULT_VAL) {
            result = BH1900NUX_OK;
        } else {
            result = BH1900NUX_WAI_ERROR;
        }
    } else {
        result = BH1900NUX_COMM_ERROR;
    }

    return result;
}

int32_t BH1900NUX::init_setting_ic(void)
{
    int32_t result;
    uint8_t config[BH1900NUX_WRITE_DATA_SIZE];
    uint8_t rst_val = BH1900NUX_SW_RESET_VAL;

    result = write(BH1900NUX_SW_RESET, &rst_val, sizeof(rst_val));
    if (result == BH1900NUX_OK) {
        config[0] = (uint8_t)(BH1900NUX_CONFIG_VAL >> 8);
        config[1] = (uint8_t)BH1900NUX_CONFIG_VAL;
        (void)write(BH1900NUX_CONFIG, config, sizeof(config));
    } else {
        result = BH1900NUX_COMM_ERROR;
    }

    return result;
}

uint32_t BH1900NUX::setting_wait(uint16_t config)
{
    int32_t meas_time_tbl[BH1900NUX_CONFIG_WT_MODE_NUM] = {
            BH1900NUX_CONFIG_WT_X16_MAX,
            BH1900NUX_CONFIG_WT_X4_MAX,
            BH1900NUX_CONFIG_WT_X1_MAX,
            BH1900NUX_CONFIG_WT_X0_5_MAX
            };
    uint32_t meas_time;

    config   &= BH1900NUX_CONFIG_WT_MASK;
    meas_time = meas_time_tbl[config];

    return meas_time;
}

int32_t BH1900NUX::write(uint8_t reg, uint8_t *data, int32_t size)
{
    int32_t result;

    Wire.beginTransmission(Address);
    (void)Wire.write(reg);
    (void)Wire.write(data, (uint8_t)size);
    result = Wire.endTransmission();
    if (result == 0) {
        result = BH1900NUX_OK;
    } else {
        result = BH1900NUX_COMM_ERROR;
    }

    return result;
}

int32_t BH1900NUX::read(uint8_t reg, uint8_t *data, int32_t size)
{
    int32_t result;
    uint8_t cnt;
    uint8_t receive_size;

    Wire.beginTransmission(Address);
    (void)Wire.write(reg);
    result = Wire.endTransmission(false);
    if (result == 0) {
        receive_size = Wire.requestFrom(Address, (uint8_t)size);
        if (receive_size == size) {
            for (cnt = 0; cnt < size; cnt++) {
                data[cnt] = Wire.read();
            }
            result = BH1900NUX_OK;
        } else {
            result = BH1900NUX_COMM_ERROR;
        }
    } else {
        result = BH1900NUX_COMM_ERROR;
    }

    return result;
}